import asyncio
import json
from typing import Callable, Dict, Any
from websockets.client import connect, WebSocketClientProtocol
from .service_registry import ServiceRegistry, Service

GATEWAY_URL = "wss://agentlabs.up.railway.app"


class Agent:
    """Agent class used to register services and deploy the agent on Weavel."""

    def __init__(self):
        self.service_registry = ServiceRegistry()

    def service(self, name: str, description: str):
        def decorator(func: Callable):
            self.service_registry.register(name, description, func)
            return func

        return decorator

    async def handle_message(
        self, message: Dict[str, Any], ws: WebSocketClientProtocol
    ):
        if message["type"] == "LIST_SERVICES":
            response = self.service_registry.list()
            await ws.send(json.dumps(response))

        elif message["type"] == "RUN_SERVICE":
            service_name = message["service_name"]
            args = message["args"]
            service: Service = self.service_registry.get(service_name)
            if service:
                result = await service.func(**args)
                await ws.send(json.dumps({"result": result}))

    async def connect_to_gateway(self, token: str):
        async with connect(f"{GATEWAY_URL}/ws/agent/{token}") as ws:
            while True:
                message = await ws.recv()
                data = json.loads(message)
                await self.handle_message(data, ws)

    def run(self, token: str):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.connect_to_gateway(token))
